/*
 * Copyright 2003 - 2009 The eFaps Team
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * Revision:        $Rev: 4824 $
 * Last Changed:    $Date: 2010-06-19 10:01:25 -0500 (Sat, 19 Jun 2010) $
 * Last Changed By: $Author: jan.moxter $
 */

package org.efaps.ui.wicket.resources;

import org.apache.wicket.Application;
import org.apache.wicket.SharedResources;
import org.efaps.ci.CIAdminProgram;

/**
 * The Resource for a XSL.
 *
 * @author The eFaps Team
 * @version $Id: XSLResource.java 4824 2010-06-19 15:01:25Z jan.moxter $
 */
public class XSLResource
    extends AbstractEFapsResource
{
    /**
     * Needed for serialization.
     */
    private static final long serialVersionUID = 1L;

    /**
     * Constructor settingt the Name of the Type in the SuperClass.
     *
     * @param _name Name of the Resource
     */
    public XSLResource(final String _name)
    {
        super(_name, CIAdminProgram.XSL.getType().getName());
    }

    /**
     * method to retrieve a XSLResource. It will check if it allready exist and
     * return the existung one. If no Resource exists a new one will be created.
     *
     * @param _name Name of the Resource
     * @return the XSLResource related to the Name
     */
    public static XSLResource get(final String _name)
    {
        final SharedResources sharedResources =
                        Application.get().getSharedResources();

        XSLResource resource = (XSLResource) sharedResources.get(_name);

        if (resource == null) {
            resource = new XSLResource(_name);
            sharedResources.add(_name, null, resource);
        }
        return resource;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    protected AbstractEFapsResourceStream setNewResourceStream()
    {
        return new XSLResourceStream();
    }

    /**
     * Class for a ResourceStream for XSL.
     */
    protected class XSLResourceStream
        extends AbstractEFapsResourceStream
    {
        /**
         * Needed for serialization.
         */
        private static final long serialVersionUID = 1L;

        /**
         * {@inheritDoc}
         */
        public String getContentType()
        {
            return null;
        }

    }
}
